package cn.kdan.compdf.config;

import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.JwtTokenEnum;
import org.springframework.stereotype.Component;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jws;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.security.Keys;

import javax.crypto.SecretKey;
import java.util.Date;

/**
 * @author Bob 2022-10-11
 **/
@Component
public class JwtConfig {

    /**
     * 加密
     */
    public static String generateJwt(LoginUserDTO user) {

        // 加密
        byte[] keyBytes = AuthEnum.SECRET.getBytes();
        // 获得密钥对象
        SecretKey key = Keys.hmacShaKeyFor(keyBytes);

        String token = Jwts.builder()
                .setHeaderParam("typ", "JWT") //令牌类型
                .setIssuedAt(new Date()) //签发时间
                .setExpiration(new Date(System.currentTimeMillis() + AuthEnum.EXPIRE * 1000 * 60 * 60)) //过期时间
                .claim(JwtTokenEnum.ID, user.getId())
                .claim(JwtTokenEnum.USERNAME, user.getUsername())
                .claim(JwtTokenEnum.FULL_NAME, user.getFullName())
                .claim(JwtTokenEnum.TENANT_ID, user.getTenantId())
                .signWith(key, SignatureAlgorithm.HS256)
                .compact();
        return token;
    }

    /**
     * 解析jwt，一定要捕获异常
     * @param jwtToken
     * @return
     */
    public static LoginUserDTO checkJwt(String jwtToken) {

        Jws<Claims> claimsJws = Jwts.parser().setSigningKey(AuthEnum.SECRET.getBytes()).parseClaimsJws(jwtToken);
        // map
        Claims claims = claimsJws.getBody();
        Long id = claims.get(JwtTokenEnum.ID,Long.class);
        String username = claims.get(JwtTokenEnum.USERNAME,String.class);
        String fullName = claims.get(JwtTokenEnum.FULL_NAME,String.class);
        Long tenantId = claims.get(JwtTokenEnum.TENANT_ID,Long.class);

        return LoginUserDTO.builder()
                .id(id)
                .username(username)
                .tenantId(tenantId)
                .fullName(fullName)
                .build();
    }

/*    public static void main(String[] args) {
//        String s = generateJwt(LoginUserDTO.builder().id(123L).username("kdan").tenantId("1").fullName("kk").build());
//        System.out.println(s);
        try {
            String token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjE2NjU5NzExMjYsImV4cCI6MTY2NTk3NDcyNiwiaWQiOjEyMywidXNlcm5hbWUiOiJrZGFuIiwidGVhbmF0SWQiOiIxIn0.LzPDSixl0RBqYCD60K3FXa8RBEFm9gbYboB6RqINGvM";
            LoginUserDTO loginUserDTO = checkJwt(token);
            System.out.println(loginUserDTO);
        } catch (Exception e) {
            throw new BusinessException("登录过期");
        }
    }*/

}
