package cn.kdan.compdf.exception;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.enums.BackgroundErrorEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ZhouQiang 2022/7/5
 */
@RestControllerAdvice(basePackages = "cn.kdan.compdf")
@Slf4j
public class GlobalExceptionHandler {

    @ExceptionHandler(Throwable.class)
    public R<Void> handlerThrowable(Throwable t) {
        log.info("错误类：{}", t.getClass());
        log.error(t.getMessage(), t);
        return R.error();
    }

    @ExceptionHandler(CommonException.class)
    public R<Void> handlerThrowable(CommonException e) {
        log.error(e.getMessage(), e);
//        log.error(e.getMessage());
        return R.error(e.getCode(), e.getMessage());
    }

    @ExceptionHandler(BusinessException.class)
    public R<Void> handlerThrowable(BusinessException e) {
        log.error(e.getMessage(), e);
//        log.error(e.getMessage());
        return R.error(e.getCode(), e.getMessage());
    }

    @ExceptionHandler({BindException.class, MethodArgumentNotValidException.class})
    public R<Map<String, String>> handlerBindException(Exception e) {
        log.error("参数校验失败", e);
        BindingResult bindingResult = e instanceof BindException ? ((BindException) e).getBindingResult() : ((MethodArgumentNotValidException) e).getBindingResult();
        Map<String, String> errorData = bindingResult.getFieldErrors().stream()
                .filter(f -> f.getDefaultMessage() != null)
                .collect(Collectors.toMap(FieldError::getField, FieldError::getDefaultMessage, (k1, k2) -> k1));
        return R.error(BackgroundErrorEnum.PARAM_VALIDATE_ERROR, errorData);
    }
}
