package cn.kdan.compdf.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.kdan.compdf.dto.AddFreePlanDTO;
import cn.kdan.compdf.dto.Asset;
import cn.hutool.core.date.DateUtil;
import cn.kdan.compdf.config.ExcelWidthStyleStrategy;
import cn.kdan.compdf.dto.PlanStatisticsDTO;
import cn.kdan.compdf.entity.BackgroundUserBalance;
import cn.kdan.compdf.entity.BackgroundUserRemain;
import cn.kdan.compdf.mapper.BackgroundUserBalanceMapper;
import cn.kdan.compdf.service.BackgroundUserBalanceService;
import cn.kdan.compdf.service.BackgroundUserRemainService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.utils.MyLocalDateUtil;
import cn.kdan.compdf.vo.BackgroundUserBalanceVO;
import cn.kdan.compdf.vo.BackgroundUserPackageBalanceVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.assertj.core.util.Lists;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2022-10-14
 */
@Service
@Slf4j
public class BackgroundUserBalanceServiceImpl extends ServiceImpl<BackgroundUserBalanceMapper, BackgroundUserBalance> implements BackgroundUserBalanceService {


    private final StringRedisTemplate redisTemplate;

    private final BackgroundUserService userService;
    private final BackgroundUserRemainService backgroundUserRemainService;

    public BackgroundUserBalanceServiceImpl(StringRedisTemplate redisTemplate, BackgroundUserService userService, BackgroundUserRemainService backgroundUserRemainService) {
        this.redisTemplate = redisTemplate;
        this.userService = userService;
        this.backgroundUserRemainService = backgroundUserRemainService;
    }

    @Override
    public IPage<BackgroundUserBalance> page(BackgroundUserBalance query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    /**
     * 获取账户套餐余额
     *
     * @param userId 用户id
     * @return 账户套餐余额
     */
    @Override
    public BackgroundUserPackageBalanceVO getPackageBalance(Long userId) {
        LambdaQueryWrapper<BackgroundUserRemain> eq = new LambdaQueryWrapper<BackgroundUserRemain>()
                .eq(BackgroundUserRemain::getUserId, userId);
        BackgroundUserRemain backgroundUserRemain = backgroundUserRemainService.getOne(eq);
        return BackgroundUserPackageBalanceVO.builder()
                .usedFiles(backgroundUserRemain.getUsedFiles()).remainingFiles(backgroundUserRemain.getTotalFiles()).build();
    }

    /**
     * 获取账户消费充值记录
     *
     * @param userId 用户id
     * @param page   当前页
     * @param size   当前页行数
     * @return 账户消费充值记录
     */
    @Override
    public IPage<BackgroundUserBalanceVO> getBalanceRecordList(Long userId, Integer page, Integer size, Integer timeZone) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>()
                .eq(BackgroundUserBalance::getUserId, userId)
                .orderByDesc(BackgroundUserBalance::getDate);
        Page<BackgroundUserBalance> backgroundUserBalancePage = this.baseMapper.selectPage(new Page<>(page, size), eq);
        // 处理返回数据
        Page<BackgroundUserBalanceVO> backgroundUserBalanceVOPage = new Page<>();
        backgroundUserBalanceVOPage.setCurrent(backgroundUserBalancePage.getCurrent());
        backgroundUserBalanceVOPage.setPages(backgroundUserBalancePage.getPages());
        backgroundUserBalanceVOPage.setSize(backgroundUserBalancePage.getSize());
        backgroundUserBalanceVOPage.setTotal(backgroundUserBalancePage.getTotal());
        List<BackgroundUserBalanceVO> records = Lists.newArrayList();
        if (CollectionUtil.isNotEmpty(backgroundUserBalancePage.getRecords())) {
            backgroundUserBalancePage.getRecords().forEach(c -> {
                c.setDate(MyLocalDateUtil.timeZoneConversion(c.getDate(), timeZone));
                BackgroundUserBalanceVO backgroundUserBalanceVO = new BackgroundUserBalanceVO();
                BeanUtil.copyProperties(c, backgroundUserBalanceVO);
                if (c.getChangeType() == 1) {
                    backgroundUserBalanceVO.setBalanceChange("+" + c.getBalanceChange());
                } else {
                    backgroundUserBalanceVO.setBalanceChange("-" + c.getBalanceChange());
                }
                records.add(backgroundUserBalanceVO);
            });
        }
        backgroundUserBalanceVOPage.setRecords(records);
        return backgroundUserBalanceVOPage;
    }

    /**
     * 账户消费充值记录导出
     *
     * @param userId   用户id
     * @param response response
     */
    @Override
    public void exportBalanceRecordList(Integer isDecs, Long userId, HttpServletResponse response) {
        LambdaQueryWrapper<BackgroundUserBalance> eq = new LambdaQueryWrapper<BackgroundUserBalance>()
                .eq(BackgroundUserBalance::getUserId, userId)
                .orderByDesc(BackgroundUserBalance::getCreateDate);
        List<BackgroundUserBalance> backgroundUserBalanceList = this.baseMapper.selectList(eq);
        // 处理返回数据
        List<BackgroundUserBalanceVO> backgroundUserBalanceVOList = new ArrayList<>();
        if (CollectionUtil.isNotEmpty(backgroundUserBalanceList)) {
            backgroundUserBalanceList.forEach(c -> {
                BackgroundUserBalanceVO backgroundUserBalanceVO = new BackgroundUserBalanceVO();
                BeanUtil.copyProperties(c, backgroundUserBalanceVO);
                if (c.getChangeType() == 1) {
                    backgroundUserBalanceVO.setBalanceChange("+" + c.getBalanceChange());
                } else {
                    backgroundUserBalanceVO.setBalanceChange("-" + c.getBalanceChange());
                }
                backgroundUserBalanceVOList.add(backgroundUserBalanceVO);
            });
        }
        if (isDecs == 0) {
            Collections.reverse(backgroundUserBalanceVOList);
        }
        // 写入响应导出
        try {
            String fileName = URLEncoder.encode("Processing Files Balance (" + DateUtil.date().toString().replaceAll(":", "-") + ")", "UTF-8").replaceAll("\\+", "%20");
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream(), BackgroundUserBalanceVO.class)
                    .registerWriteHandler(new ExcelWidthStyleStrategy())
                    .autoCloseStream(Boolean.FALSE)
                    .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                    .sheet("Processing Files Balance")
                    .doWrite(backgroundUserBalanceVOList);

        } catch (Exception e) {
            // 重置response
            response.reset();
            response.setContentType("application/json");
            response.setCharacterEncoding("utf-8");
            Map<String, String> map = MapUtils.newHashMap();
            map.put("status", "failure");
            map.put("message", "下载文件失败" + e.getMessage());
            try {
                response.getWriter().println(JSON.toJSONString(map));
            } catch (IOException ioException) {
                ioException.printStackTrace();
            }
        }
    }

    @Override
    public void planSync(List<PlanStatisticsDTO> list) {
        assert list != null : "Dashboard同步数据为空";
        final String PLAN_SYNC_KEY = "planSyncKey:" + list.get(0).getSyncTime();
        Boolean b = redisTemplate.hasKey(PLAN_SYNC_KEY);
        assert b != null;
        if (!b) {
            log.info("开始数据同步");
            redisTemplate.opsForValue().setIfAbsent(PLAN_SYNC_KEY, "", 1, TimeUnit.DAYS);
            try {
                List<BackgroundUserBalance> userBalanceList = new ArrayList<>();
                list.forEach(item -> {
                    Long userId = userService.getUserId(item.getTenantId());
                    // 最新的plan剩余额度
                    Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(item.getTenantId());
                    BackgroundUserBalance backgroundUserBalance = new BackgroundUserBalance();
                    backgroundUserBalance.setUserId(userId);
                    backgroundUserBalance.setTenantId(item.getTenantId());
                    backgroundUserBalance.setDate(LocalDateTime.now());
                    backgroundUserBalance.setDescriptionType(9);
                    backgroundUserBalance.setDescription("Files consumed");  // TODO 枚举值
                    backgroundUserBalance.setChangeType(2);
                    backgroundUserBalance.setRemainingFiles((int) (newestPlanRemainingFiles - item.getPlan()));
                    backgroundUserBalance.setBalanceChange(item.getPlan().intValue());
                    userBalanceList.add(backgroundUserBalance);
                });
                this.saveBatch(userBalanceList);
            } catch (Exception e) {
                redisTemplate.delete(PLAN_SYNC_KEY);
                throw e;
            }
        } else {
            log.error("当前时间节点已同步过：{}" + PLAN_SYNC_KEY);
        }
    }

    @Override
    public void freePlanSync(AddFreePlanDTO freePlanDTO) {
        Integer freeAsset = freePlanDTO.getFreeAsset();
        List<Asset> assets = freePlanDTO.getAssets();
        for (Asset asset : assets) {
            // 最新的plan剩余额度
            Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(asset.getTenantId());
            BackgroundUserBalance userBalance = new BackgroundUserBalance();
            userBalance.setTenantId(asset.getTenantId());
            userBalance.setUserId(userService.getUserId(asset.getTenantId()));
            userBalance.setDate(LocalDateTime.now());
            userBalance.setDescriptionType(9);
            userBalance.setDescription("Files consumed"); // TODO 枚举值
            userBalance.setRemainingFiles(newestPlanRemainingFiles - asset.getAsset());
            userBalance.setBalanceChange(asset.getAsset());
            userBalance.setChangeType(3);

            this.save(userBalance);

            newestPlanRemainingFiles = getNewestPlanRemainingFiles(asset.getTenantId());
            userBalance.setId(null);
            userBalance.setDate(userBalance.getDate().plusSeconds(1L));
            userBalance.setDescriptionType(3);
            userBalance.setDescription("Free files"); // TODO 枚举值
            userBalance.setRemainingFiles(newestPlanRemainingFiles + freeAsset);
            userBalance.setBalanceChange(freeAsset);
            userBalance.setChangeType(1);
            this.save(userBalance);
        }
        List<Asset> cleanFreeAssets = freePlanDTO.getCleanFreeAssets();
        for (Asset cleanFreeAsset : cleanFreeAssets) {
            // 最新的plan剩余额度
            Integer newestPlanRemainingFiles = getNewestPlanRemainingFiles(cleanFreeAsset.getTenantId());
            BackgroundUserBalance userBalance = new BackgroundUserBalance();
            userBalance.setTenantId(cleanFreeAsset.getTenantId());
            userBalance.setUserId(userService.getUserId(cleanFreeAsset.getTenantId()));
            userBalance.setDate(LocalDateTime.now());
            userBalance.setDescriptionType(9);
            userBalance.setDescription("Files consumed"); // TODO 枚举值
            userBalance.setRemainingFiles(newestPlanRemainingFiles - cleanFreeAsset.getAsset());
            userBalance.setBalanceChange(cleanFreeAsset.getAsset());
            userBalance.setChangeType(3);

            this.save(userBalance);

            Integer newestPlanAddFreeNum = this.baseMapper.selectNewestPlanAddFreeNum(cleanFreeAsset.getTenantId());
            backgroundUserRemainService.cleanFreeAssetTotal(cleanFreeAsset.getTenantId(), newestPlanAddFreeNum);

        }

    }

    /**
     * 获取最新的plan资产余额
     *
     * @param tenantId tenantId
     * @return 最新的plan资产余额
     */
    private Integer getNewestPlanRemainingFiles(Long tenantId) {
        Integer newestPlanRemainingFiles = this.baseMapper.selectNewestPlanRemainingFiles(tenantId);
        if (ObjectUtil.isEmpty(newestPlanRemainingFiles)) {
            newestPlanRemainingFiles = 0;
        }
        return newestPlanRemainingFiles;
    }

}
