package cn.kdan.compdf.service.impl;

import cn.kdan.compdf.base.R;
import cn.kdan.compdf.config.JwtConfig;
import cn.kdan.compdf.dto.ChangePasswordDTO;
import cn.kdan.compdf.dto.LoginDTO;
import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.entity.BackgroundUser;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import cn.kdan.compdf.mapper.BackgroundUserMapper;
import cn.kdan.compdf.service.BackgroundUserProjectService;
import cn.kdan.compdf.service.BackgroundUserService;
import cn.kdan.compdf.utils.MD5Util;
import cn.kdan.compdf.utils.TokenUtil;
import cn.kdan.compdf.vo.LoginVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * @author comPDF-Kit WPH 2022-10-09
 */
@Slf4j
@Service
public class BackgroundUserServiceImpl extends ServiceImpl<BackgroundUserMapper, BackgroundUser> implements BackgroundUserService {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private BackgroundUserProjectService backgroundUserProjectService;

    @Override
    public IPage<BackgroundUser> page(BackgroundUser query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    @Override
    public R<LoginVO> login(LoginDTO loginDTO) {
        BackgroundUser backgroundUser = this.baseMapper.selectOne(new LambdaQueryWrapper<BackgroundUser>().eq(BackgroundUser::getUsername, loginDTO.getUsername()));
        if (null == backgroundUser) {
            throw new BusinessException("账号不存在");
        }
        //转换成MD5密码
        String md5Password = MD5Util.MD5Encode(loginDTO.getPassword());
        if (!md5Password.equals(backgroundUser.getPassword())) {
            throw new BusinessException("密码错误");
        }
        String token = JwtConfig.generateJwt(LoginUserDTO
                .builder()
                .id(backgroundUser.getId())
                .username(backgroundUser.getUsername())
                .fullName(backgroundUser.getFullName())
                .tenantId(backgroundUser.getTenantId())
                .build());
        log.info("用户登录成功：username：{}，time：{}", loginDTO.getUsername(), new Date());

        // 设置token过期时间
        redisTemplate.opsForValue().set(AuthEnum.USER_TOKEN_REDIS + token, token, AuthEnum.EXPIRE, TimeUnit.HOURS);

        //更新登录用户的登陆时间
        if (backgroundUser.getFirstLoginTime() == null) {
            // 首次登录创建默认项目
            backgroundUserProjectService.defaultProject(backgroundUser.getId(),backgroundUser.getTenantId());
            backgroundUser.setFirstLoginTime(LocalDateTime.now());
        }
        backgroundUser.setLastLoginTime(new Date());
        baseMapper.updateById(backgroundUser);

        LoginVO loginVO = new LoginVO();
        loginVO.setId(backgroundUser.getId());
        loginVO.setUsername(backgroundUser.getUsername());
        loginVO.setFirstLoginTime(backgroundUser.getFirstLoginTime().toString());
        loginVO.setToken(token);
        return R.ok(loginVO);
    }

    @Override
    public void logout(String token) {
        redisTemplate.delete(AuthEnum.USER_TOKEN_REDIS + token);
    }

    @Override
    public void changePassword(ChangePasswordDTO changePasswordDTO, String token) {
        if (changePasswordDTO.getOldPassword().equals(changePasswordDTO.getNewPassword())){
            throw new BusinessException(ResponseEnum.OLDPWD_EQUALS_NEWPWD);
        }
        // 获取当前登录用户
        LoginUserDTO requestHeader = TokenUtil.getRequestHeader();
        BackgroundUser backgroundUser = this.baseMapper.selectOne(new LambdaQueryWrapper<BackgroundUser>().eq(BackgroundUser::getId, requestHeader.getId()));
        //转换成MD5密码
        String oldPassword = MD5Util.MD5Encode(changePasswordDTO.getOldPassword());
        if (!oldPassword.equals(backgroundUser.getPassword())) {
            throw new BusinessException("原密码错误");
        }
        //转换成MD5密码
        String newPassword = MD5Util.MD5Encode(changePasswordDTO.getNewPassword());
        backgroundUser.setPassword(newPassword);
        baseMapper.updateById(backgroundUser);
        // 退出登录
        redisTemplate.delete(AuthEnum.USER_TOKEN_REDIS + token);
    }

    @Override
    public Long getUserId(Long tenantId) {
        return this.baseMapper.selectOne(new LambdaQueryWrapper<BackgroundUser>().select(BackgroundUser::getId).eq(BackgroundUser::getTenantId,tenantId)).getId();
    }

    @Override
    public LocalDateTime getUSerFirstLogin(Long tenantId) {
        return this.baseMapper.selectOne(new LambdaQueryWrapper<BackgroundUser>().select(BackgroundUser::getFirstLoginTime)
                .eq(BackgroundUser::getTenantId, tenantId)).getFirstLoginTime();
    }

}
